#include "types.h"
#include "riscv.h"
#include "defs.h"
#include "date.h"
#include "param.h"
#include "memlayout.h"
#include "spinlock.h"
#include "proc.h"
#include "sysinfo.h"

// 全局变量，存储 trace 掩码
int trace_mask = 0;

uint64 sys_exit(void) {
  int n;
  if (argint(0, &n) < 0) return -1;
  exit(n);
  return 0;  // not reached
}

uint64 sys_getpid(void) { return myproc()->pid; }

uint64 sys_fork(void) { return fork(); }

uint64 sys_wait(void) {
  uint64 p;
  if (argaddr(0, &p) < 0) return -1;
  return wait(p);
}

uint64 sys_sbrk(void) {
  int addr;
  int n;

  if (argint(0, &n) < 0) return -1;
  addr = myproc()->sz;
  if (growproc(n) < 0) return -1;
  return addr;
}

uint64 sys_sleep(void) {
  int n;
  uint ticks0;

  if (argint(0, &n) < 0) return -1;
  acquire(&tickslock);
  ticks0 = ticks;
  while (ticks - ticks0 < n) {
    if (myproc()->killed) {
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

uint64 sys_kill(void) {
  int pid;

  if (argint(0, &pid) < 0) return -1;
  return kill(pid);
}

// return how many clock tick interrupts have occurred
// since start.
uint64 sys_uptime(void) {
  uint xticks;

  acquire(&tickslock);
  xticks = ticks;
  release(&tickslock);
  return xticks;
}

uint64 sys_rename(void) {
  char name[16];
  int len = argstr(0, name, MAXPATH);
  if (len < 0) {
    return -1;
  }
  struct proc *p = myproc();
  memmove(p->name, name, len);
  p->name[len] = '\0';
  return 0;
}

uint64 sys_trace(void) {
  int mask;
  if(argint(0, &mask) < 0)
    return -1;
  trace_mask = mask;  // 设置全局的 trace 掩码
  return 0;
}

// 系统调用函数
uint64 sys_sysinfo(void) {
    struct sysinfo info;
    struct proc *p = myproc();
    uint64 addr;  // 用户空间地址

    if (argaddr(0, &addr) < 0)  // 获取用户传入的参数
        return -1;

    // 获取空闲内存量
    info.freemem = kfreemem();  // 需要自己实现kfreemem获取内存
    info.nproc = get_unused_procs();  // 获取UNUSED状态的进程数，需要实现该函数

    // 将结构体信息复制到用户空间
    if (copyout(p->pagetable, addr, (char *)&info, sizeof(info)) < 0)
        return -1;

    return 0;
}


uint64 sys_wait_sched(void) {
    uint64 runable_time_addr, running_time_addr, sleep_time_addr;
    int *runable_time, *running_time, *sleep_time;

    // 获取用户传入的参数并验证
    if (argaddr(0, &runable_time_addr) < 0 ||  
        argaddr(1, &running_time_addr) < 0 ||  
        argaddr(2, &sleep_time_addr) < 0) 
        return -1;

    // 将地址转换为指针
    runable_time = (int *)runable_time_addr;
    running_time = (int *)running_time_addr;
    sleep_time = (int *)sleep_time_addr;

    // 调用 wait_sched 函数并返回结果
    return wait_sched(runable_time, running_time, sleep_time);
}

uint64 sys_set_priority(void) {
    int priority, pid;

    // 获取并验证系统调用参数
    if (argint(0, &priority) < 0 || argint(1, &pid) < 0)
        return -1;

    // 调用 set_priority 函数并返回结果
    return set_priority(priority, pid);
}

